@@ -381,6 +381,37 @@ static int x509_enc_valid_to(struct x509_generator_opts *opts,
 	return lc_x509_cert_set_valid_to(cert, (time64_t)val);
 }
 
+static int x509_enc_valid_days(struct x509_generator_opts *opts,
+			       const char *opt_optarg)
+{
+	struct lc_x509_certificate *cert = &opts->cert;
+	time64_t now;
+	unsigned long long val;
+	int ret;
+
+	val = strtoull(opt_optarg, NULL, 10);
+	if (val == ULLONG_MAX)
+		return -ERANGE;
+
+	now = time(NULL);
+	if (now == ((time_t)-1))
+		return -errno;
+
+	/* Read data is in days -> convert to seconds */
+	val *= 24; /* hours */
+	val *= 60; /* minutes */
+	val *= 60; /* seconds */
+
+	/* Turn val into the "valid-to" time */
+	val += (unsigned long long)now;
+
+	CKINT(lc_x509_cert_set_valid_from(cert, now));
+	CKINT(lc_x509_cert_set_valid_to(cert, (time64_t)val));
+
+out:
+	return ret;
+}
+
 static int x509_enc_subject_cn(struct x509_generator_opts *opts,
 			       const char *opt_optarg)
 {
@@ -718,6 +749,26 @@ out:
 	return ret;
 }
 
+static int x509_check_data(struct x509_generator_opts *opts)
+{
+	struct lc_x509_certificate *cert = &opts->cert;
+	int ret = 0;
+
+	if (!opts->raw_serial) {
+#define LC_X509_RANDOM_SERIAL_SIZE 8
+		opts->raw_serial = calloc(1, LC_X509_RANDOM_SERIAL_SIZE);
+		CKNULL(opts->raw_serial, -ENOMEM);
+		opts->raw_serial_size = LC_X509_RANDOM_SERIAL_SIZE;
+		CKINT(lc_rng_generate(lc_seeded_rng, NULL, 0, opts->raw_serial,
+				      opts->raw_serial_size));
+		CKINT(lc_x509_cert_set_serial(cert, opts->raw_serial,
+					      opts->raw_serial_size));
+	}
+
+out:
+	return ret;
+}
+
 static void x509_generator_usage(void)
 {
 	fprintf(stderr, "\nLeancrypto X.509 Certificate Generator\n");
@@ -732,35 +783,34 @@ static void x509_generator_usage(void)
 	fprintf(stderr, "\n\tOptions for X.509 cryptographic aspects:\n");
 	fprintf(stderr,
 		"\t   --sk-file <FILE>\t\tFile with secret key used for signature\n");
-	fprintf(stderr, "\t\t\t\t\t\tInput when key is available\n"),
-		fprintf(stderr, "\t\t\t\t\t\t(MUST be DER),\n");
-	fprintf(stderr, "\t\t\t\t\t\toutput with --create-keypair\n");
+	fprintf(stderr, "\t\t\t\t\tInput when key is available (MUST be\n"),
+		fprintf(stderr, "\t\t\t\t\tDER),\n");
+	fprintf(stderr, "\t\t\t\t\toutput with --create-keypair\n");
 	fprintf(stderr,
 		"\t   --pk-file <FILE>\t\tFile with public key used for signature\n");
-	fprintf(stderr, "\t\t\t\t\t\tInput when key is available\n"),
-		fprintf(stderr,
-			"\t\t\t\t\t\tto generate cert (MUST be plain),\n");
-	fprintf(stderr, "\t\t\t\t\t\toutput with --create-keypair\n");
+	fprintf(stderr, "\t\t\t\t\tInput when key is available to generate\n"),
+		fprintf(stderr, "\t\t\t\t\tcert (MUST be DER),\n");
+	fprintf(stderr, "\t\t\t\t\toutput with --create-keypair\n");
 	fprintf(stderr,
 		"\t   --key-type <TYPE>\t\tInput keys are of given type\n");
+	fprintf(stderr, "\t\t\t\t\tQuery available types with \"?\"\n");
 	fprintf(stderr,
 		"\t   --create-keypair <TYPE>\tCreate key pair of given type\n");
-	fprintf(stderr, "\t\t\t\t\t\tNOTE: generated keys are written\n");
-	fprintf(stderr, "\t\t\t\t\t\tto file\n");
+	fprintf(stderr, "\t\t\t\t\tNOTE: generated keys are written to file\n");
 	fprintf(stderr,
 		"\t   --x509-signer <FILE>\t\tX.509 certificate of signer\n");
-	fprintf(stderr, "\t\t\t\t\t\tIf not set, create a self-signed\n");
-	fprintf(stderr, "\t\t\t\t\t\tcertificate\n");
+	fprintf(stderr, "\t\t\t\t\tIf not set, create a self-signed\n");
+	fprintf(stderr, "\t\t\t\t\tcertificate\n");
 	fprintf(stderr,
-		"\t   --signer-sk-file <FILE>\t\tFile with signer secret\n");
+		"\t   --signer-sk-file <FILE>\tFile with signer secret\n");
 	fprintf(stderr, "\t   --x509-cert <FILE>\t\tCertificate for signing\n");
 
 	fprintf(stderr, "\n\tOptions for X.509 meta data:\n");
 	fprintf(stderr, "\t   --eku <FLAG>\t\t\tSet Extended Key Usage flag\n");
-	fprintf(stderr, "\t\t\t\t\t\tQuery available flags with \"?\"\n");
+	fprintf(stderr, "\t\t\t\t\tQuery available flags with \"?\"\n");
 
 	fprintf(stderr, "\t   --keyusage <FLAG>\t\tSet Key Usage flag\n");
-	fprintf(stderr, "\t\t\t\t\t\tQuery available flags with \"?\"\n");
+	fprintf(stderr, "\t\t\t\t\tQuery available flags with \"?\"\n");
 
 	fprintf(stderr,
 		"\t   --ca\t\t\t\tSet CA basic constraint with criticality\n");
@@ -769,10 +819,13 @@ static void x509_generator_usage(void)
 		"\t   --san-ip <IP> \t\tSet SAN IP address (IPv4 or IPv6)\n");
 	fprintf(stderr, "\t   --skid\t\t\tSet SKID (in hex form)\n");
 	fprintf(stderr, "\t   --akid\t\t\tSet AKID (in hex form)\n");
-	fprintf(stderr, "\t\t\t\t\t\tAKID only used without X.509\n");
-	fprintf(stderr, "\t\t\t\t\t\tsigner being specified\n");
+	fprintf(stderr, "\t\t\t\t\tAKID only used without X.509 signer\n");
+	fprintf(stderr, "\t\t\t\t\tbeing specified\n");
 	fprintf(stderr, "\t   --valid-from\t\t\tSet start time\n");
 	fprintf(stderr, "\t   --valid-to\t\t\tSet end time\n");
+	fprintf(stderr,
+		"\t   --valid-days\t\t\tSet validity time in days from\n");
+	fprintf(stderr, "\t\t\t\t\tfrom today\n");
 	fprintf(stderr,
 		"\t   --serial <VALUE>\t\tSet serial numer (in hex form)\n");
 
@@ -825,9 +878,9 @@ static void x509_generator_usage(void)
 		"\t   --check-valid-to <EPOCH time>\tcheck validity of time\n");
 	fprintf(stderr,
 		"\t   --check-eku <EKU>\t\tmatch extended key usage (use KEY_EKU_*\n");
-	fprintf(stderr,
-		"\t   --check-keyusage <EKU>\tmatch key usage (use KEY_USAGE_*\n");
 	fprintf(stderr, "\t\t\t\t\tflags)\n");
+	fprintf(stderr,
+		"\t   --check-keyusage <EKU>\tmatch key usage (use KEY_USAGE_* flags)\n");
 	fprintf(stderr, "\t   --check-san-dns <NAME>\tmatch SAN DNS\n");
 	fprintf(stderr, "\t   --check-san-ip <IP-Hex>\tmatch SAN IP\n");
 	fprintf(stderr, "\t   --check-skid <HEX>\t\tmatch subject key ID\n");
@@ -835,7 +888,9 @@ static void x509_generator_usage(void)
 
 	fprintf(stderr, "\t   --data-file <FILE>\t\tFile with data to sign\n");
 
-	fprintf(stderr, "\n\t-h  --help\t\t\tPrint this help text\n");
+	fprintf(stderr, "\n\t-h --help\t\t\tPrint this help text\n");
+	fprintf(stderr,
+		"\n\t-v --version\t\t\tPrint version and acceleration support\n");
 }
 
 static void x509_generator_version(void)
@@ -880,6 +935,7 @@ int main(int argc, char *argv[])
 					      { "akid", 1, 0, 0 },
 					      { "valid-from", 1, 0, 0 },
 					      { "valid-to", 1, 0, 0 },
+					      { "valid-days", 1, 0, 0 },
 					      { "serial", 1, 0, 0 },
 
 					      { "subject-cn", 1, 0, 0 },
@@ -965,7 +1021,7 @@ int main(int argc, char *argv[])
 				CKINT_LOG(lc_x509_pkey_name_to_algorithm(
 						  optarg,
 						  &ws->parsed_opts.in_key_type),
-					  "Key type paring failure\n");
+					  "Key type unknown\n");
 				break;
 			/* create-keypair */
 			case 6:
@@ -989,13 +1045,13 @@ int main(int argc, char *argv[])
 			case 9:
 				CKINT_LOG(x509_enc_eku(&ws->parsed_opts,
 						       optarg),
-					  "EKU parsing error\n");
+					  "EKU unknown\n");
 				break;
 			/* keyusage */
 			case 10:
 				CKINT_LOG(x509_enc_keyusage(&ws->parsed_opts,
 							    optarg),
-					  "Key usage parsing error\n");
+					  "Key usage unknown\n");
 				break;
 			/* ca */
 			case 11:
@@ -1039,188 +1095,201 @@ int main(int argc, char *argv[])
 							    optarg),
 					  "Set valid to\n");
 				break;
-			/* serial */
+			/* valid-days */
 			case 18:
+				/*
+				 * There is deliberately no control whether the
+				 * caller used valid-from/to and valid-days at
+				 * the same time - it is his fault if he uses
+				 * conflicting information. Whatever comes last
+				 * is used to set the time.
+				 */
+				CKINT_LOG(x509_enc_valid_days(&ws->parsed_opts,
+							      optarg),
+					  "Set valid days\n");
+				break;
+			/* serial */
+			case 19:
 				CKINT_LOG(x509_enc_serial(&ws->parsed_opts,
 							  optarg),
 					  "Set serial\n");
 				break;
 
 			/* subject-cn */
-			case 19:
+			case 20:
 				CKINT_LOG(x509_enc_subject_cn(&ws->parsed_opts,
 							      optarg),
 					  "Subject CN parsing error\n");
 				break;
 			/* subject-email */
-			case 20:
+			case 21:
 				CKINT_LOG(x509_enc_subject_email(
 						  &ws->parsed_opts, optarg),
 					  "Subject email parsing error\n");
 				break;
 			/* subject-ou */
-			case 21:
+			case 22:
 				CKINT_LOG(x509_enc_subject_ou(&ws->parsed_opts,
 							      optarg),
 					  "Subject OU parsing error\n");
 				break;
 			/* subject-o */
-			case 22:
+			case 23:
 				CKINT_LOG(x509_enc_subject_o(&ws->parsed_opts,
 							     optarg),
 					  "Subject O parsing error\n");
 				break;
 			/* subject-st */
-			case 23:
+			case 24:
 				CKINT(x509_enc_subject_st(&ws->parsed_opts,
 							  optarg));
 				break;
 			/* subject-c */
-			case 24:
+			case 25:
 				CKINT(x509_enc_subject_c(&ws->parsed_opts,
 							 optarg));
 				break;
 
 			/* issuer-cn */
-			case 25:
+			case 26:
 				CKINT(x509_enc_issuer_cn(&ws->parsed_opts,
 							 optarg));
 				break;
 			/* issuer-email */
-			case 26:
+			case 27:
 				CKINT(x509_enc_issuer_email(&ws->parsed_opts,
 							    optarg));
 				break;
 			/* issuer-ou */
-			case 27:
+			case 28:
 				CKINT(x509_enc_issuer_ou(&ws->parsed_opts,
 							 optarg));
 				break;
 			/* issuer-o */
-			case 28:
+			case 29:
 				CKINT(x509_enc_issuer_o(&ws->parsed_opts,
 							optarg));
 				break;
 			/* issuer-st */
-			case 29:
+			case 30:
 				CKINT(x509_enc_issuer_st(&ws->parsed_opts,
 							 optarg));
 				break;
 			/* issuer-c */
-			case 30:
+			case 31:
 				CKINT(x509_enc_issuer_c(&ws->parsed_opts,
 							optarg));
 				break;
 
 			/* print */
-			case 31:
+			case 32:
 				ws->parsed_opts.print_x509 = 1;
 				break;
 			/* noout */
-			case 32:
+			case 33:
 				ws->parsed_opts.noout = 1;
 				break;
 			/* print-x509 */
-			case 33:
+			case 34:
 				ws->parsed_opts.print_x509_cert = optarg;
 				break;
 
 			/* check-ca */
-			case 34:
+			case 35:
 				checker_opts->check_ca = 1;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-ca-conformant */
-			case 35:
+			case 36:
 				checker_opts->check_ca_conformant = 1;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-time */
-			case 36:
+			case 37:
 				checker_opts->check_time = 1;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-issuer-cn */
-			case 37:
+			case 38:
 				checker_opts->issuer_cn = optarg;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-subject-cn */
-			case 38:
+			case 39:
 				checker_opts->subject_cn = optarg;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-noselfsigned */
-			case 39:
+			case 40:
 				checker_opts->check_no_selfsigned = 1;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-valid-from */
-			case 40:
+			case 41:
 				checker_opts->valid_from =
 					strtoull(optarg, NULL, 10);
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-valid-to */
-			case 41:
+			case 42:
 				checker_opts->valid_to =
 					strtoull(optarg, NULL, 10);
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-eku */
-			case 42:
+			case 43:
 				checker_opts->eku =
 					(unsigned int)strtoul(optarg, NULL, 10);
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-san-dns */
-			case 43:
+			case 44:
 				checker_opts->san_dns = optarg;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-san-ip */
-			case 44:
+			case 45:
 				checker_opts->san_ip = optarg;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-skid */
-			case 45:
+			case 46:
 				checker_opts->skid = optarg;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-akid */
-			case 46:
+			case 47:
 				checker_opts->akid = optarg;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-noca */
-			case 47:
+			case 48:
 				checker_opts->check_no_ca = 1;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-selfsigned */
-			case 48:
+			case 49:
 				checker_opts->check_selfsigned = 1;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-rootca */
-			case 49:
+			case 50:
 				checker_opts->check_root_ca = 1;
 				ws->parsed_opts.checker = 1;
 				break;
 			/* check-keyusage */
-			case 50:
+			case 51:
 				checker_opts->keyusage =
 					(unsigned int)strtoul(optarg, NULL, 10);
 				ws->parsed_opts.checker = 1;
 				break;
 
 			/* data-file */
-			case 51:
+			case 52:
 				ws->parsed_opts.data_file = optarg;
 				break;
 			/* x509-cert */
-			case 52:
+			case 53:
 				ws->parsed_opts.x509_cert_file = optarg;
 				break;
 			}
@@ -1245,6 +1314,8 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	CKINT(x509_check_data(&ws->parsed_opts));
+
 	if (ws->parsed_opts.print_x509_cert) {
 		if (ws->parsed_opts.x509_signer_file)
 			CKINT(x509_enc_set_signer(&ws->parsed_opts));
