@@ -19,11 +19,11 @@
 
 #include "asn1_encoder.h"
 #include "asym_key.h"
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
 #include "lc_pkcs7_generator.h"
 #include "lc_x509_generator.h"
-#include "pkcs7.asn1.h"
-#include "pkcs7_aa.asn1.h"
+#include "pkcs7_asn1.h"
+#include "pkcs7_aa_asn1.h"
 #include "oid_registry.h"
 #include "ret_checkers.h"
 #include "visibility.h"
@@ -578,18 +578,18 @@ static int pkcs7_set_time(uint8_t *data, size_t *avail_datalen, uint8_t *tag)
 	/* GenTime: YYYYMMDDHHMMSSZ */
 #define X509_GENTIM_SIZE 15
 	/*
-	 * Add 2 trailing bytes to shut up the stupid compiler check that cannot
+	 * Add 7 trailing bytes to shut up the stupid compiler check that cannot
 	 * detect the modulo operation.
 	 */
-	char datestr[X509_GENTIM_SIZE + 2];
-	struct tm *time_detail;
+	char datestr[X509_GENTIM_SIZE + 7];
+	struct lc_tm time_detail;
 	time64_t timeval;
 	int ret;
 
 	CKINT(lc_get_time(&timeval));
 
 	/* UTC time */
-	time_detail = gmtime((const time_t *)&timeval);
+	CKINT(lc_gmtime(timeval, &time_detail));
 
 	/*
 	 * The value is the time since EPOCH for 2050-01-01
@@ -607,9 +607,8 @@ static int pkcs7_set_time(uint8_t *data, size_t *avail_datalen, uint8_t *tag)
 #pragma GCC diagnostic pop
 		CKINT(x509_sufficient_size(avail_datalen, X509_GENTIM_SIZE));
 		snprintf(datestr, sizeof(datestr), "%.4d%.2d%.2d%.2d%.2d%.2dZ",
-			 time_detail->tm_year + 1900, time_detail->tm_mon + 1,
-			 time_detail->tm_mday, time_detail->tm_hour,
-			 time_detail->tm_min, time_detail->tm_sec);
+			 time_detail.year, time_detail.month, time_detail.day,
+			 time_detail.hour, time_detail.min, time_detail.sec);
 
 		memcpy(data, datestr, X509_GENTIM_SIZE);
 		*avail_datalen -= X509_GENTIM_SIZE;
@@ -617,9 +616,9 @@ static int pkcs7_set_time(uint8_t *data, size_t *avail_datalen, uint8_t *tag)
 	} else {
 		CKINT(x509_sufficient_size(avail_datalen, X509_UCTTIM_SIZE));
 		snprintf(datestr, sizeof(datestr), "%02d%02d%02d%02d%02d%02dZ",
-			 time_detail->tm_year % 100, time_detail->tm_mon + 1,
-			 time_detail->tm_mday, time_detail->tm_hour,
-			 time_detail->tm_min, time_detail->tm_sec);
+			 time_detail.year % 100, time_detail.month,
+			 time_detail.day, time_detail.hour, time_detail.min,
+			 time_detail.sec);
 
 		memcpy(data, datestr, X509_UCTTIM_SIZE);
 		*avail_datalen -= X509_UCTTIM_SIZE;
@@ -640,7 +639,7 @@ static int pkcs7_hash_data(uint8_t *digest, size_t *digest_size,
 	int ret;
 
 	/* Digest the message [RFC5652 5.4] */
-	lc_hash_init(hash_ctx);
+	CKINT(lc_hash_init(hash_ctx));
 	CKINT(x509_set_digestsize(digest_size, hash_ctx));
 
 	lc_hash_update(hash_ctx, pkcs7->data, pkcs7->data_len);
@@ -820,7 +819,7 @@ int pkcs7_sig_note_set_of_authattrs_enc(void *context, uint8_t *data,
 		return -EBADMSG;
 	}
 
-	lc_hash_init(hash_ctx);
+	CKINT(lc_hash_init(hash_ctx));
 	ctx->authattrs_digest_size = sizeof(ctx->authattrs_digest);
 	CKINT(x509_set_digestsize(&ctx->authattrs_digest_size, hash_ctx));
 
