@@ -31,13 +31,13 @@
 #include "x509_cert_generator.h"
 #include "x509_cert_parser.h"
 
-#include "x509.asn1.h"
-#include "x509_akid.asn1.h"
-#include "x509_basic_constraints.asn1.h"
-#include "x509_eku.asn1.h"
-#include "x509_keyusage.asn1.h"
-#include "x509_san.asn1.h"
-#include "x509_skid.asn1.h"
+#include "x509_asn1.h"
+#include "x509_akid_asn1.h"
+#include "x509_basic_constraints_asn1.h"
+#include "x509_eku_asn1.h"
+#include "x509_keyusage_asn1.h"
+#include "x509_san_asn1.h"
+#include "x509_skid_asn1.h"
 
 int x509_concatenate_bit_string(uint8_t **dst_data, size_t *dst_avail_datalen,
 				const uint8_t *src_data, size_t src_datalen)
@@ -137,7 +137,7 @@ int x509_eku_enc(void *context, uint8_t *data, size_t *avail_datalen,
 		CKINT(OID_to_data(OID_id_kp_OCSPSigning, &oid_data,
 				  &oid_datalen));
 	} else {
-		return -EINVAL;
+		CKRET(1, -EINVAL);
 	}
 
 	bin2print_debug(oid_data, oid_datalen, stdout, "OID");
@@ -175,12 +175,12 @@ int x509_basic_constraints_ca_enc(void *context, uint8_t *data,
 	struct x509_generate_context *ctx = context;
 	const struct lc_x509_certificate *cert = ctx->cert;
 	const struct lc_public_key *pub = &cert->pub;
+	int ret = 0;
 
 	(void)tag;
 
 	if (x509_pathlen_unprocessed(ctx)) {
-		if (*avail_datalen < 1)
-			return -EOVERFLOW;
+		CKRET(*avail_datalen < 1, -EOVERFLOW);
 
 		if (pub->basic_constraint & LC_KEY_CA)
 			*data = ASN1_TRUE;
@@ -194,7 +194,8 @@ int x509_basic_constraints_ca_enc(void *context, uint8_t *data,
 		printf_debug("Setting CA: %u\n", *data);
 	}
 
-	return 0;
+out:
+	return ret;
 }
 
 /*
@@ -206,12 +207,12 @@ int x509_basic_constraints_pathlen_enc(void *context, uint8_t *data,
 	struct x509_generate_context *ctx = context;
 	const struct lc_x509_certificate *cert = ctx->cert;
 	const struct lc_public_key *pub = &cert->pub;
+	int ret = 0;
 
 	(void)tag;
 
 	if (x509_pathlen_unprocessed(ctx)) {
-		if (*avail_datalen < 1)
-			return -EOVERFLOW;
+		CKRET(*avail_datalen < 1, -EOVERFLOW);
 
 		*data = pub->ca_pathlen;
 		*avail_datalen -= 1;
@@ -219,7 +220,8 @@ int x509_basic_constraints_pathlen_enc(void *context, uint8_t *data,
 		ctx->pathlen_processed = *data;
 	}
 
-	return 0;
+out:
+	return ret;
 }
 
 int x509_name_unprocessed(const struct lc_x509_certificate_name *name,
@@ -398,13 +400,13 @@ int x509_san_dns_enc(void *context, uint8_t *data, size_t *avail_datalen,
 {
 	struct x509_generate_context *ctx = context;
 	const struct lc_x509_certificate *cert = ctx->cert;
+	int ret = 0;
 
 	(void)tag;
 
 	if (cert->san_dns_len &&
 	    !(ctx->san_processed & X509_SAN_DNS_PROCESSED)) {
-		if (*avail_datalen < cert->san_dns_len)
-			return -EOVERFLOW;
+		CKRET(*avail_datalen < cert->san_dns_len, -EOVERFLOW);
 
 		memcpy(data, cert->san_dns, cert->san_dns_len);
 		*avail_datalen -= cert->san_dns_len;
@@ -412,7 +414,8 @@ int x509_san_dns_enc(void *context, uint8_t *data, size_t *avail_datalen,
 		ctx->san_processed |= X509_SAN_DNS_PROCESSED;
 	}
 
-	return 0;
+out:
+	return ret;
 }
 
 /*
@@ -423,12 +426,12 @@ int x509_san_ip_enc(void *context, uint8_t *data, size_t *avail_datalen,
 {
 	struct x509_generate_context *ctx = context;
 	const struct lc_x509_certificate *cert = ctx->cert;
+	int ret = 0;
 
 	(void)tag;
 
 	if (cert->san_ip_len && !(ctx->san_processed & X509_SAN_IP_PROCESSED)) {
-		if (*avail_datalen < cert->san_ip_len)
-			return -EOVERFLOW;
+		CKRET(*avail_datalen < cert->san_ip_len, -EOVERFLOW);
 
 		memcpy(data, cert->san_ip, cert->san_ip_len);
 		*avail_datalen -= cert->san_ip_len;
@@ -436,7 +439,8 @@ int x509_san_ip_enc(void *context, uint8_t *data, size_t *avail_datalen,
 		ctx->san_processed |= X509_SAN_IP_PROCESSED;
 	}
 
-	return 0;
+out:
+	return ret;
 }
 
 static inline int x509_keyusage_unprocessed(struct x509_generate_context *ctx,
@@ -693,6 +697,7 @@ int x509_extension_critical_enc(void *context, uint8_t *data,
 	const struct lc_public_key *pub = &cert->pub;
 #define X509_EXTENSION_UNSET 0xffffffff
 	unsigned int val = X509_EXTENSION_UNSET;
+	int ret = 0;
 
 	(void)tag;
 
@@ -714,8 +719,7 @@ int x509_extension_critical_enc(void *context, uint8_t *data,
 	if (val == X509_EXTENSION_UNSET)
 		return 0;
 
-	if (*avail_datalen < 1)
-		return -EOVERFLOW;
+	CKRET(*avail_datalen < 1, -EOVERFLOW);
 
 	if (val)
 		*data = ASN1_TRUE;
@@ -724,7 +728,8 @@ int x509_extension_critical_enc(void *context, uint8_t *data,
 
 	*avail_datalen -= 1;
 
-	return 0;
+out:
+	return ret;
 }
 
 int x509_process_extension_enc(void *context, uint8_t *data,
@@ -816,6 +821,7 @@ int x509_note_algorithm_OID_enc(void *context, uint8_t *data,
 	const uint8_t *oid_data = NULL;
 	enum OID oid;
 	size_t oid_datalen = 0;
+	const char *alg;
 	int ret = 0;
 
 	(void)tag;
@@ -830,8 +836,8 @@ int x509_note_algorithm_OID_enc(void *context, uint8_t *data,
 
 	CKINT(lc_x509_sig_type_to_oid(pkey_algo, &oid));
 	CKINT(OID_to_data(oid, &oid_data, &oid_datalen));
-	bin2print_debug(oid_data, oid_datalen, stdout,
-			lc_x509_sig_type_to_name(pkey_algo));
+	CKINT(lc_x509_sig_type_to_name(pkey_algo, &alg))
+	bin2print_debug(oid_data, oid_datalen, stdout, alg);
 
 	if (oid_datalen) {
 		CKINT(x509_sufficient_size(avail_datalen, oid_datalen));
@@ -1039,13 +1045,13 @@ int x509_set_uct_time_enc(void *context, uint8_t *data, size_t *avail_datalen,
 	/* UCTTime: YYMMDDHHMMSSZ */
 #define X509_UCTTIM_SIZE 13
 	/*
-	 * Add 2 trailing bytes to shut up the stupid compiler check that cannot
+	 * Add 6 trailing bytes to shut up the stupid compiler check that cannot
 	 * detect the modulo operation.
 	 */
-	char datestr[X509_UCTTIM_SIZE + 2];
+	char datestr[X509_UCTTIM_SIZE + 6];
 	struct x509_generate_context *ctx = context;
 	const struct lc_x509_certificate *cert = ctx->cert;
-	struct tm *time_detail;
+	struct lc_tm time_detail;
 	time64_t tmp_time;
 	int ret;
 
@@ -1079,11 +1085,10 @@ int x509_set_uct_time_enc(void *context, uint8_t *data, size_t *avail_datalen,
 	/*
 	 * NOTE: The caller is assumed to have set the time in UTC time.
 	 */
-	time_detail = gmtime((time_t *)&ctx->time_to_set);
-	snprintf(datestr, sizeof(datestr), "%02d%02d%02d%02d%02d%02dZ",
-		 time_detail->tm_year % 100, time_detail->tm_mon + 1,
-		 time_detail->tm_mday, time_detail->tm_hour,
-		 time_detail->tm_min, time_detail->tm_sec);
+	CKINT(lc_gmtime(ctx->time_to_set, &time_detail));
+	snprintf(datestr, sizeof(datestr), "%.2d%.2d%.2d%.2d%.2d%.2dZ",
+		 time_detail.year % 100, time_detail.month, time_detail.day,
+		 time_detail.hour, time_detail.min, time_detail.sec);
 
 	memcpy(data, datestr, X509_UCTTIM_SIZE);
 	*avail_datalen -= X509_UCTTIM_SIZE;
@@ -1100,10 +1105,14 @@ int x509_set_gen_time_enc(void *context, uint8_t *data, size_t *avail_datalen,
 {
 	/* GenTime: YYYYMMDDHHMMSSZ */
 #define X509_GENTIM_SIZE 15
-	char datestr[X509_GENTIM_SIZE + 1];
+	/*
+	 * Add 6 trailing bytes to shut up the stupid compiler check that cannot
+	 * detect the modulo operation.
+	 */
+	char datestr[X509_GENTIM_SIZE + 6];
 	struct x509_generate_context *ctx = context;
 	const struct lc_x509_certificate *cert = ctx->cert;
-	struct tm *time_detail;
+	struct lc_tm time_detail;
 	time64_t tmp_time;
 	int ret;
 
@@ -1134,24 +1143,10 @@ int x509_set_gen_time_enc(void *context, uint8_t *data, size_t *avail_datalen,
 		return 0;
 	}
 
-	/*
-	 * NOTE: The caller is assumed to have set the time in UTC time.
-	 *
-	 * WARNING On 32 bit systems without support for 64-bit time, this will
-	 * wrap and set the wrong time as time_t is 32 bit on those systems.
-	 *
-	 * In this case, be safe than sorry and return an error.
-	 */
-#ifndef __LP64__
-	if (sizeof(time_t) != 8)
-		return -EOVERFLOW;
-#endif
-
-	time_detail = gmtime((time_t *)&ctx->time_to_set);
+	CKINT(lc_gmtime(ctx->time_to_set, &time_detail));
 	snprintf(datestr, sizeof(datestr), "%.4d%.2d%.2d%.2d%.2d%.2dZ",
-		 time_detail->tm_year + 1900, time_detail->tm_mon + 1,
-		 time_detail->tm_mday, time_detail->tm_hour,
-		 time_detail->tm_min, time_detail->tm_sec);
+		 time_detail.year & 0x1FFF, time_detail.month, time_detail.day,
+		 time_detail.hour, time_detail.min, time_detail.sec);
 
 	memcpy(data, datestr, X509_GENTIM_SIZE);
 	*avail_datalen -= X509_GENTIM_SIZE;
@@ -1171,6 +1166,7 @@ int x509_note_not_before_enc(void *context, uint8_t *data,
 {
 	struct x509_generate_context *ctx = context;
 	const struct lc_x509_certificate *cert = ctx->cert;
+	int ret = 0;
 
 	(void)data;
 	(void)avail_datalen;
@@ -1179,13 +1175,14 @@ int x509_note_not_before_enc(void *context, uint8_t *data,
 	/* Sanity check */
 	if (ctx->time_to_set != cert->valid_from) {
 		printf_debug("Parser error: validity not before wrong\n");
-		return -EFAULT;
+		CKRET(1, -EFAULT);
 	}
 
 	/* Now, reset the flag to allow for the next round */
 	ctx->time_already_set = 0;
 
-	return 0;
+out:
+	return ret;
 }
 
 /*
@@ -1196,6 +1193,7 @@ int x509_note_not_after_enc(void *context, uint8_t *data, size_t *avail_datalen,
 {
 	struct x509_generate_context *ctx = context;
 	const struct lc_x509_certificate *cert = ctx->cert;
+	int ret = 0;
 
 	(void)data;
 	(void)avail_datalen;
@@ -1204,13 +1202,14 @@ int x509_note_not_after_enc(void *context, uint8_t *data, size_t *avail_datalen,
 	/* Sanity check */
 	if (ctx->time_to_set != cert->valid_to) {
 		printf_debug("Parser error: validity not after wrong\n");
-		return -EFAULT;
+		CKRET(1, -EFAULT);
 	}
 
 	/* Now, reset the flag to allow for the next round */
 	ctx->time_already_set = 0;
 
-	return 0;
+out:
+	return ret;
 }
 
 int x509_extract_key_data_enc(void *context, uint8_t *data,
@@ -1284,6 +1283,9 @@ LC_INTERFACE_FUNCTION(int, lc_x509_cert_encode,
 
 	ws->gctx.cert = x509;
 
+	/* Ensure that validity makes sense */
+	CKRET(x509->valid_from > x509->valid_to, -EINVAL);
+
 	/*
 	 * Attempt to encode the certificate
 	 */
@@ -1345,21 +1347,31 @@ out:
 LC_INTERFACE_FUNCTION(int, lc_x509_get_signature_size_from_sk, size_t *siglen,
 		      const struct lc_x509_key_data *keys)
 {
-	if (!siglen || !keys)
-		return -EINVAL;
-	return public_key_signature_size(siglen, keys->sig_type);
+	int ret;
+
+	CKNULL(siglen, -EINVAL);
+	CKNULL(keys, -EINVAL);
+
+	CKINT(public_key_signature_size(siglen, keys->sig_type));
+
+out:
+	return ret;
 }
 
 LC_INTERFACE_FUNCTION(int, lc_x509_get_signature_size_from_cert, size_t *siglen,
 		      const struct lc_x509_certificate *cert)
 {
 	const struct lc_public_key *pub;
+	int ret;
 
-	if (!siglen || !cert)
-		return -EINVAL;
+	CKNULL(siglen, -EINVAL);
+	CKNULL(cert, -EINVAL);
 
 	pub = &cert->pub;
-	return public_key_signature_size(siglen, pub->pkey_algo);
+	CKINT(public_key_signature_size(siglen, pub->pkey_algo));
+
+out:
+	return ret;
 }
 
 LC_INTERFACE_FUNCTION(int, lc_x509_signature_gen, uint8_t *sig_data,
@@ -1376,8 +1388,7 @@ LC_INTERFACE_FUNCTION(int, lc_x509_signature_gen, uint8_t *sig_data,
 	CKNULL(m, -EINVAL);
 
 	if (prehash_algo) {
-		if (mlen > LC_SHA_MAX_SIZE_DIGEST)
-			return -EOVERFLOW;
+		CKRET(mlen > LC_SHA_MAX_SIZE_DIGEST, -EOVERFLOW);
 
 		memcpy(sig.digest, m, mlen);
 		sig.digest_size = mlen;
@@ -1434,6 +1445,31 @@ LC_INTERFACE_FUNCTION(int, lc_x509_cert_set_signer,
 	CKNULL(dparam, -EINVAL);
 	CKINT(lc_x509_cert_set_akid(signed_x509, dparam, paramlen));
 
+	/* If no validity is set, force the signers validity */
+	if (!signed_x509->valid_to)
+		signed_x509->valid_to = signer_x509->valid_to;
+	if (!signed_x509->valid_from)
+		signed_x509->valid_from = signer_x509->valid_from;
+
+	/*
+	 * If the valid-to value is older than the signer's valid-from
+	 * then we cannot sign.
+	 */
+	CKRET(signed_x509->valid_to < signer_x509->valid_from, -EINVAL);
+	/*
+	 * If the valid-from value is newer than the signer's valid-to
+	 * then we cannot sign.
+	 */
+	CKRET(signed_x509->valid_from > signer_x509->valid_to, -EINVAL);
+
+	/* Limit the valid-to to the maximum of the signer */
+	if (signed_x509->valid_to > signer_x509->valid_to)
+		signed_x509->valid_to = signer_x509->valid_to;
+
+	/* Limit the valid-from to the minimum of the signer */
+	if (signed_x509->valid_from < signer_x509->valid_from)
+		signed_x509->valid_from = signer_x509->valid_from;
+
 	CKINT(asym_set_signer(signed_x509, signer_key_data, signer_x509));
 
 out:
