@@ -35,10 +35,10 @@
 #include "math_helper.h"
 #include "ret_checkers.h"
 #include "small_stack_support.h"
-#include "x509.asn1.h"
-#include "x509_akid.asn1.h"
-#include "x509_basic_constraints.asn1.h"
-#include "x509_eku.asn1.h"
+#include "x509_asn1.h"
+#include "x509_akid_asn1.h"
+#include "x509_basic_constraints_asn1.h"
+#include "x509_eku_asn1.h"
 #include "x509_cert_generator.h"
 #include "x509_cert_parser.h"
 
