@@ -29,7 +29,7 @@
 #ifndef X25519_SCALARMULT_H
 #define X25519_SCALARMULT_H
 
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
 
 #ifdef __cplusplus
 extern "C" {
