@@ -20,7 +20,7 @@
 #ifndef X25519_SCALARMULT_C_H
 #define X25519_SCALARMULT_C_H
 
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
 
 #ifdef __cplusplus
 extern "C" {
