@@ -20,7 +20,7 @@
 #ifndef SEEDED_RNG_H
 #define SEEDED_RNG_H
 
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
 
 #ifdef __cplusplus
 extern "C" {
