@@ -20,7 +20,7 @@
 #include <windows.h>
 #include <bcrypt.h>
 
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
 #include "math_helper.h"
 #include "seeded_rng.h"
 
