@@ -17,21 +17,21 @@
  * DAMAGE.
  */
 
-#ifndef SHA2_512_ARMv8_H
-#define SHA2_512_ARMv8_H
+#ifndef SHA2_256_ARMv8_H
+#define SHA2_256_ARMv8_H
 
-#include "ext_headers.h"
-#include "lc_sha512.h"
+#include "ext_headers_internal.h"
+#include "lc_sha256.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-void sha512_block_neon(struct lc_sha512_state *c, const uint8_t *p, size_t num);
-void sha512_block_armv8ce(struct lc_sha512_state *c, const uint8_t *p,
+void sha256_block_neon(struct lc_sha256_state *c, const uint8_t *p, size_t num);
+void sha256_block_armv8ce(struct lc_sha256_state *c, const uint8_t *p,
 			  size_t num);
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* SHA2_512_ARMv8_H */
+#endif /* SHA2_256_ARMv8_H */
