@@ -19,10 +19,14 @@
 
 #include "build_bug_on.h"
 #include "bitshift.h"
+#include "compare.h"
 #include "conv_be_le.h"
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
+#include "fips_mode.h"
+#include "keccak_internal.h"
 #include "lc_sha3.h"
 #include "lc_memset_secure.h"
+#include "lc_status.h"
 #include "math_helper.h"
 #include "sha3_c.h"
 #include "sha3_common.h"
@@ -159,32 +163,6 @@ static inline void keccakp_1600(uint64_t s[25])
 
 /************************ Raw Keccak Sponge Operations *************************/
 
-#if __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
-
-/*
- * This function works on both endianesses, but since it has more code than
- * the little endian code base, there is a special case for little endian.
- */
-static inline void sha3_fill_state_bytes(uint64_t *state, const uint8_t *in,
-					 size_t byte_offset, size_t inlen)
-{
-	sponge_fill_state_bytes(state, in, byte_offset, inlen, le_bswap64);
-}
-
-#elif __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-
-static inline void sha3_fill_state_bytes(uint64_t *state, const uint8_t *in,
-					 size_t byte_offset, size_t inlen)
-{
-	uint8_t *_state = (uint8_t *)state;
-
-	xor_64(_state + byte_offset, in, inlen);
-}
-
-#else
-#error "Endianess not defined"
-#endif
-
 static void keccak_c_permutation(void *state, unsigned int rounds)
 {
 	(void)rounds;
@@ -245,16 +223,24 @@ void sha3_224_init_common(void *_state)
 	ctx->padding = 0x06;
 }
 
-static void sha3_224_init(void *_state)
+static int sha3_224_init_nocheck(void *_state)
 {
 	struct lc_sha3_224_state *ctx = _state;
-	static int tested = 0;
 
 	if (!ctx)
-		return;
+		return -EINVAL;
 
-	sha3_224_selftest_common(lc_sha3_224_c, &tested, "SHA3-224 C");
 	sha3_224_init_common(_state);
+
+	return 0;
+}
+
+static int sha3_224_init(void *_state)
+{
+	sha3_224_selftest_common(lc_sha3_224_c);
+	LC_SELFTEST_COMPLETED(LC_ALG_STATUS_SHA3);
+
+	return sha3_224_init_nocheck(_state);
 }
 
 size_t sha3_224_digestsize(void *_state)
@@ -277,16 +263,24 @@ void sha3_256_init_common(void *_state)
 	ctx->padding = 0x06;
 }
 
-static void sha3_256_init(void *_state)
+static int sha3_256_init_nocheck(void *_state)
 {
 	struct lc_sha3_256_state *ctx = _state;
-	static int tested = 0;
 
 	if (!ctx)
-		return;
+		return -EINVAL;
 
-	sha3_256_selftest_common(lc_sha3_256_c, &tested, "SHA3-256 C");
 	sha3_256_init_common(_state);
+
+	return 0;
+}
+
+static int sha3_256_init(void *_state)
+{
+	sha3_256_selftest_common(lc_sha3_256_c);
+	LC_SELFTEST_COMPLETED(LC_ALG_STATUS_SHA3);
+
+	return sha3_256_init_nocheck(_state);
 }
 
 size_t sha3_256_digestsize(void *_state)
@@ -309,16 +303,24 @@ void sha3_384_init_common(void *_state)
 	ctx->padding = 0x06;
 }
 
-static void sha3_384_init(void *_state)
+static int sha3_384_init_nocheck(void *_state)
 {
 	struct lc_sha3_384_state *ctx = _state;
-	static int tested = 0;
 
 	if (!ctx)
-		return;
+		return -EINVAL;
 
-	sha3_384_selftest_common(lc_sha3_384_c, &tested, "SHA3-384 C");
 	sha3_384_init_common(_state);
+
+	return 0;
+}
+
+static int sha3_384_init(void *_state)
+{
+	sha3_384_selftest_common(lc_sha3_384_c);
+	LC_SELFTEST_COMPLETED(LC_ALG_STATUS_SHA3);
+
+	return sha3_384_init_nocheck(_state);
 }
 
 size_t sha3_384_digestsize(void *_state)
@@ -341,16 +343,24 @@ void sha3_512_init_common(void *_state)
 	ctx->padding = 0x06;
 }
 
-static void sha3_512_init(void *_state)
+static int sha3_512_init_nocheck(void *_state)
 {
 	struct lc_sha3_512_state *ctx = _state;
-	static int tested = 0;
 
 	if (!ctx)
-		return;
+		return -EINVAL;
 
-	sha3_512_selftest_common(lc_sha3_512_c, &tested, "SHA3-512 C");
 	sha3_512_init_common(_state);
+
+	return 0;
+}
+
+static int sha3_512_init(void *_state)
+{
+	sha3_512_selftest_common(lc_sha3_512_c);
+	LC_SELFTEST_COMPLETED(LC_ALG_STATUS_SHA3);
+
+	return sha3_512_init_nocheck(_state);
 }
 
 size_t sha3_512_digestsize(void *_state)
@@ -373,16 +383,24 @@ void shake_128_init_common(void *_state)
 	ctx->padding = 0x1f;
 }
 
-static void shake_128_init(void *_state)
+static int shake_128_init_nocheck(void *_state)
 {
 	struct lc_shake_128_state *ctx = _state;
-	static int tested = 0;
 
 	if (!ctx)
-		return;
+		return -EINVAL;
 
-	shake128_selftest_common(lc_shake128_c, &tested, "SHAKE128 C");
 	shake_128_init_common(_state);
+
+	return 0;
+}
+
+static int shake_128_init(void *_state)
+{
+	shake128_selftest_common(lc_shake128_c);
+	LC_SELFTEST_COMPLETED(LC_ALG_STATUS_SHAKE);
+
+	return shake_128_init_nocheck(_state);
 }
 
 void shake_256_init_common(void *_state)
@@ -393,22 +411,66 @@ void shake_256_init_common(void *_state)
 		return;
 
 	sha3_ctx_init(_state);
-	ctx->r = LC_SHA3_256_SIZE_BLOCK;
-	ctx->rword = LC_SHA3_256_SIZE_BLOCK / sizeof(uint64_t);
+	BUILD_BUG_ON(LC_SHA3_256_SIZE_BLOCK != LC_SHAKE_256_SIZE_BLOCK);
+	ctx->r = LC_SHAKE_256_SIZE_BLOCK;
+	ctx->rword = LC_SHAKE_256_SIZE_BLOCK / sizeof(uint64_t);
 	ctx->digestsize = 0;
 	ctx->padding = 0x1f;
 }
 
-static void shake_256_init(void *_state)
+static int shake_256_init_nocheck(void *_state)
 {
 	struct lc_sha3_256_state *ctx = _state;
-	static int tested = 0;
+
+	if (!ctx)
+		return -EINVAL;
+
+	shake_256_init_common(_state);
+
+	return 0;
+}
+
+static int shake_256_init(void *_state)
+{
+	shake256_selftest_common(lc_shake256_c);
+	LC_SELFTEST_COMPLETED(LC_ALG_STATUS_SHAKE);
+
+	return shake_256_init_nocheck(_state);
+}
+
+void shake_512_init_common(void *_state)
+{
+	struct lc_sha3_512_state *ctx = _state;
 
 	if (!ctx)
 		return;
 
-	shake256_selftest_common(lc_shake256_c, &tested, "SHAKE256 C");
-	shake_256_init_common(_state);
+	sha3_ctx_init(_state);
+	BUILD_BUG_ON(LC_SHA3_512_SIZE_BLOCK != LC_SHAKE_512_SIZE_BLOCK);
+	ctx->r = LC_SHAKE_512_SIZE_BLOCK;
+	ctx->rword = LC_SHAKE_512_SIZE_BLOCK / sizeof(uint64_t);
+	ctx->digestsize = 0;
+	ctx->padding = 0x1f;
+}
+
+static int shake_512_init_nocheck(void *_state)
+{
+	struct lc_sha3_512_state *ctx = _state;
+
+	if (!ctx)
+		return -EINVAL;
+
+	shake_512_init_common(_state);
+
+	return 0;
+}
+
+static int shake_512_init(void *_state)
+{
+	shake512_selftest_common(lc_shake512_c);
+	LC_SELFTEST_COMPLETED(LC_ALG_STATUS_SHAKE512);
+
+	return shake_512_init_nocheck(_state);
 }
 
 void cshake_256_init_common(void *_state)
@@ -425,16 +487,24 @@ void cshake_256_init_common(void *_state)
 	ctx->padding = 0x04;
 }
 
-static void cshake_256_init(void *_state)
+static int cshake_256_init_nocheck(void *_state)
 {
 	struct lc_sha3_256_state *ctx = _state;
-	static int tested = 0;
 
 	if (!ctx)
-		return;
+		return -EINVAL;
 
-	cshake256_selftest_common(lc_cshake256_c, &tested, "cSHAKE256 C");
 	cshake_256_init_common(_state);
+
+	return 0;
+}
+
+static int cshake_256_init(void *_state)
+{
+	cshake256_selftest_common(lc_cshake256_c);
+	LC_SELFTEST_COMPLETED(LC_ALG_STATUS_CSHAKE);
+
+	return cshake_256_init_nocheck(_state);
 }
 
 void cshake_128_init_common(void *_state)
@@ -451,42 +521,24 @@ void cshake_128_init_common(void *_state)
 	ctx->padding = 0x04;
 }
 
-static void cshake_128_init(void *_state)
+static int cshake_128_init_nocheck(void *_state)
 {
 	struct lc_shake_128_state *ctx = _state;
-	static int tested = 0;
 
 	if (!ctx)
-		return;
+		return -EINVAL;
 
-	cshake128_selftest_common(lc_cshake128_c, &tested, "cSHAKE128 C");
 	cshake_128_init_common(_state);
+
+	return 0;
 }
 
-/*
- * All lc_sha3_*_state are equal except for the last entry, thus we use
- * the largest state.
- */
-static inline void sha3_fill_state(struct lc_sha3_224_state *ctx,
-				   const uint8_t *in)
+static int cshake_128_init(void *_state)
 {
-	unsigned int i;
+	cshake128_selftest_common(lc_cshake128_c);
+	LC_SELFTEST_COMPLETED(LC_ALG_STATUS_CSHAKE);
 
-	for (i = 0; i < ctx->rword; i++) {
-		ctx->state[i] ^= ptr_to_le64(in);
-		in += 8;
-	}
-}
-
-static inline void sha3_fill_state_aligned(struct lc_sha3_224_state *ctx,
-					   const uint64_t *in)
-{
-	unsigned int i;
-
-	for (i = 0; i < ctx->rword; i++) {
-		ctx->state[i] ^= le_bswap64(*in);
-		in++;
-	}
+	return cshake_128_init_nocheck(_state);
 }
 
 static void keccak_absorb(void *_state, const uint8_t *in, size_t inlen)
@@ -623,6 +675,7 @@ size_t shake_get_digestsize(void *_state)
 
 static const struct lc_hash _sha3_224_c = {
 	.init = sha3_224_init,
+	.init_nocheck = sha3_224_init_nocheck,
 	.update = keccak_absorb,
 	.final = keccak_squeeze,
 	.set_digestsize = NULL,
@@ -633,11 +686,13 @@ static const struct lc_hash _sha3_224_c = {
 	.sponge_newstate = keccak_c_newstate,
 	.sponge_rate = LC_SHA3_224_SIZE_BLOCK,
 	.statesize = sizeof(struct lc_sha3_224_state),
+	.algorithm_type = LC_ALG_STATUS_SHA3
 };
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_sha3_224_c) = &_sha3_224_c;
 
 static const struct lc_hash _sha3_256_c = {
 	.init = sha3_256_init,
+	.init_nocheck = sha3_256_init_nocheck,
 	.update = keccak_absorb,
 	.final = keccak_squeeze,
 	.set_digestsize = NULL,
@@ -648,11 +703,13 @@ static const struct lc_hash _sha3_256_c = {
 	.sponge_newstate = keccak_c_newstate,
 	.sponge_rate = LC_SHA3_256_SIZE_BLOCK,
 	.statesize = sizeof(struct lc_sha3_256_state),
+	.algorithm_type = LC_ALG_STATUS_SHA3
 };
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_sha3_256_c) = &_sha3_256_c;
 
 static const struct lc_hash _sha3_384_c = {
 	.init = sha3_384_init,
+	.init_nocheck = sha3_384_init_nocheck,
 	.update = keccak_absorb,
 	.final = keccak_squeeze,
 	.set_digestsize = NULL,
@@ -663,11 +720,13 @@ static const struct lc_hash _sha3_384_c = {
 	.sponge_newstate = keccak_c_newstate,
 	.sponge_rate = LC_SHA3_384_SIZE_BLOCK,
 	.statesize = sizeof(struct lc_sha3_384_state),
+	.algorithm_type = LC_ALG_STATUS_SHA3
 };
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_sha3_384_c) = &_sha3_384_c;
 
 static const struct lc_hash _sha3_512_c = {
 	.init = sha3_512_init,
+	.init_nocheck = sha3_512_init_nocheck,
 	.update = keccak_absorb,
 	.final = keccak_squeeze,
 	.set_digestsize = NULL,
@@ -678,11 +737,13 @@ static const struct lc_hash _sha3_512_c = {
 	.sponge_newstate = keccak_c_newstate,
 	.sponge_rate = LC_SHA3_512_SIZE_BLOCK,
 	.statesize = sizeof(struct lc_sha3_512_state),
+	.algorithm_type = LC_ALG_STATUS_SHA3
 };
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_sha3_512_c) = &_sha3_512_c;
 
 static const struct lc_hash _shake128_c = {
 	.init = shake_128_init,
+	.init_nocheck = shake_128_init_nocheck,
 	.update = keccak_absorb,
 	.final = keccak_squeeze,
 	.set_digestsize = shake_set_digestsize,
@@ -693,11 +754,13 @@ static const struct lc_hash _shake128_c = {
 	.sponge_newstate = keccak_c_newstate,
 	.sponge_rate = LC_SHAKE_128_SIZE_BLOCK,
 	.statesize = sizeof(struct lc_shake_128_state),
+	.algorithm_type = LC_ALG_STATUS_SHAKE
 };
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_shake128_c) = &_shake128_c;
 
 static const struct lc_hash _shake256_c = {
 	.init = shake_256_init,
+	.init_nocheck = shake_256_init_nocheck,
 	.update = keccak_absorb,
 	.final = keccak_squeeze,
 	.set_digestsize = shake_set_digestsize,
@@ -708,11 +771,30 @@ static const struct lc_hash _shake256_c = {
 	.sponge_newstate = keccak_c_newstate,
 	.sponge_rate = LC_SHA3_256_SIZE_BLOCK,
 	.statesize = sizeof(struct lc_sha3_256_state),
+	.algorithm_type = LC_ALG_STATUS_SHAKE
 };
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_shake256_c) = &_shake256_c;
 
+static const struct lc_hash _shake512_c = {
+	.init = shake_512_init,
+	.init_nocheck = shake_512_init_nocheck,
+	.update = keccak_absorb,
+	.final = keccak_squeeze,
+	.set_digestsize = shake_set_digestsize,
+	.get_digestsize = shake_get_digestsize,
+	.sponge_permutation = keccak_c_permutation,
+	.sponge_add_bytes = keccak_c_add_bytes,
+	.sponge_extract_bytes = keccak_c_extract_bytes,
+	.sponge_newstate = keccak_c_newstate,
+	.sponge_rate = LC_SHA3_512_SIZE_BLOCK,
+	.statesize = sizeof(struct lc_sha3_512_state),
+	.algorithm_type = LC_ALG_STATUS_SHAKE512
+};
+LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_shake512_c) = &_shake512_c;
+
 static const struct lc_hash _cshake256_c = {
 	.init = cshake_256_init,
+	.init_nocheck = cshake_256_init_nocheck,
 	.update = keccak_absorb,
 	.final = keccak_squeeze,
 	.set_digestsize = shake_set_digestsize,
@@ -723,11 +805,13 @@ static const struct lc_hash _cshake256_c = {
 	.sponge_newstate = keccak_c_newstate,
 	.sponge_rate = LC_SHA3_256_SIZE_BLOCK,
 	.statesize = sizeof(struct lc_sha3_256_state),
+	.algorithm_type = LC_ALG_STATUS_CSHAKE
 };
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_cshake256_c) = &_cshake256_c;
 
 static const struct lc_hash _cshake128_c = {
 	.init = cshake_128_init,
+	.init_nocheck = cshake_128_init_nocheck,
 	.update = keccak_absorb,
 	.final = keccak_squeeze,
 	.set_digestsize = shake_set_digestsize,
@@ -738,6 +822,7 @@ static const struct lc_hash _cshake128_c = {
 	.sponge_newstate = keccak_c_newstate,
 	.sponge_rate = LC_SHAKE_128_SIZE_BLOCK,
 	.statesize = sizeof(struct lc_shake_128_state),
+	.algorithm_type = LC_ALG_STATUS_CSHAKE
 };
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_cshake128_c) = &_cshake128_c;
 
@@ -747,5 +832,6 @@ LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_sha3_384) = &_sha3_384_c;
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_sha3_512) = &_sha3_512_c;
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_shake128) = &_shake128_c;
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_shake256) = &_shake256_c;
+LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_shake512) = &_shake512_c;
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_cshake128) = &_cshake128_c;
 LC_INTERFACE_SYMBOL(const struct lc_hash *, lc_cshake256) = &_cshake256_c;
