@@ -19,7 +19,7 @@
 
 #include "hmac_selftest.h"
 
-void hmac_sha512_selftest(const char *impl)
+int hmac_sha512_selftest(void)
 {
-	(void)impl;
+	return 0;
 }
