@@ -33,6 +33,7 @@
 
 #if (LC_HQC_TYPE == 128)
 
+LC_FIPS_RODATA_SECTION
 static const __m256i alpha_ij256_1[45] = {
 	{ 0x0010000800040002, 0x001d008000400020, 0x00cd00e80074003a,
 	  0x004c002600130087 },
@@ -125,6 +126,8 @@ static const __m256i alpha_ij256_1[45] = {
 	{ 0x009600a900df00c1, 0x001a00b900260024, 0x0060002c00640055,
 	  0x00590091003b000f }
 };
+
+LC_FIPS_RODATA_SECTION
 static const __m256i alpha_ij256_2[45] = {
 	{ 0x00b4005a002d0098, 0x008f00c900ea0075, 0x0018000c00060003,
 	  0x0000000000600030 },
@@ -222,6 +225,7 @@ static const __m256i alpha_ij256_2[45] = {
  * Coefficients of polynomial G
  * stored in 256-bit values
  **/
+LC_FIPS_RODATA_SECTION
 static const union params256 {
 	__m256i param256[2];
 	uint16_t param16[sizeof(__m256i) / sizeof(uint16_t) * 2];
@@ -232,6 +236,7 @@ static const union params256 {
 
 #elif (LC_HQC_TYPE == 192)
 
+LC_FIPS_RODATA_SECTION
 static const __m256i alpha_ij256_1[55] = {
 	{ 0x0010000800040002, 0x001d008000400020, 0x00cd00e80074003a,
 	  0x004c002600130087 },
@@ -344,6 +349,8 @@ static const __m256i alpha_ij256_1[55] = {
 	{ 0x00ac0091006700a0, 0x0037002e000f00b4, 0x005500e2006a002c,
 	  0x007c00b9002000a7 }
 };
+
+LC_FIPS_RODATA_SECTION
 static const __m256i alpha_ij256_2[55] = {
 	{ 0x00b4005a002d0098, 0x008f00c900ea0075, 0x0018000c00060003,
 	  0x009d00c000600030 },
@@ -461,6 +468,7 @@ static const __m256i alpha_ij256_2[55] = {
  * Coefficients of polynomial G
  * stored in 256-bit values
  **/
+LC_FIPS_RODATA_SECTION
 static const union params256 {
 	__m256i param256[2];
 	uint16_t param16[sizeof(__m256i) / sizeof(uint16_t) * 2];
@@ -471,6 +479,7 @@ static const union params256 {
 
 #elif (LC_HQC_TYPE == 256)
 
+LC_FIPS_RODATA_SECTION
 static const __m256i alpha_ij256_1[89] = {
 	{ 0x0010000800040002, 0x001d008000400020, 0x00cd00e80074003a,
 	  0x004c002600130087 },
@@ -651,6 +660,8 @@ static const __m256i alpha_ij256_1[89] = {
 	{ 0x002200cd00ab00e1, 0x0070001f008f00ae, 0x004600f900330025,
 	  0x00a400b90048009e }
 };
+
+LC_FIPS_RODATA_SECTION
 static const __m256i alpha_ij256_2[89] = {
 	{ 0x00b4005a002d0098, 0x008f00c900ea0075, 0x0018000c00060003,
 	  0x009d00c000600030 },
@@ -831,6 +842,8 @@ static const __m256i alpha_ij256_2[89] = {
 	{ 0x006c00d10065000b, 0x00d9008000f6006b, 0x008d00d0005a0037,
 	  0x007700a2003b00c0 }
 };
+
+LC_FIPS_RODATA_SECTION
 static const __m256i alpha_ij256_3[89] = {
 	{ 0x0025009c004e0027, 0x006a00350094004a, 0x00ee007700b500d4,
 	  0x00460023009f00c1 },
@@ -1011,6 +1024,8 @@ static const __m256i alpha_ij256_3[89] = {
 	{ 0x00a800a0004500b8, 0x00e900e400c200f3, 0x00a3000400fc00f0,
 	  0x0082008800130096 }
 };
+
+LC_FIPS_RODATA_SECTION
 static const __m256i alpha_ij256_4[89] = {
 	{ 0x0014000a0005008c, 0x005d00a000500028, 0x00000000006900ba,
 	  0x0000000000000000 },
@@ -1196,6 +1211,7 @@ static const __m256i alpha_ij256_4[89] = {
  * Coefficients of polynomial G
  * stored in 256-bit values
  **/
+LC_FIPS_RODATA_SECTION
 static const union params256 {
 	__m256i param256[4];
 	uint16_t param16[sizeof(__m256i) / sizeof(uint16_t) * 4];
@@ -1405,10 +1421,10 @@ static uint16_t compute_elp(uint16_t *sigma, const uint16_t *syndromes)
 		deg_X_sigma_p = deg_X + deg_sigma_p;
 
 		// mask1 = 0xffff if(d != 0) and 0 otherwise
-		mask1 = -((uint16_t)-d >> 15);
+		mask1 = (uint16_t)-((uint16_t)-d >> 15);
 
 		// mask2 = 0xffff if(deg_X_sigma_p > deg_sigma) and 0 otherwise
-		mask2 = -((uint16_t)(deg_sigma - deg_X_sigma_p) >> 15);
+		mask2 = (uint16_t)-((uint16_t)(deg_sigma - deg_X_sigma_p) >> 15);
 
 		// mask12 = 0xffff if the deg_sigma increased and 0 otherwise
 		mask12 = mask1 & mask2;
@@ -1474,14 +1490,14 @@ static void compute_z_poly(uint16_t *z, const uint16_t *sigma, uint16_t degree,
 	z[0] = 1;
 
 	for (i = 1; i < LC_HQC_PARAM_DELTA + 1; ++i) {
-		mask = -((uint16_t)(i - degree - 1) >> 15);
+		mask = (uint16_t)-((uint16_t)(i - degree - 1) >> 15);
 		z[i] = mask & sigma[i];
 	}
 
 	z[1] ^= syndromes[0];
 
 	for (i = 2; i <= LC_HQC_PARAM_DELTA; ++i) {
-		mask = -((uint16_t)(i - degree - 1) >> 15);
+		mask = (uint16_t)-((uint16_t)(i - degree - 1) >> 15);
 		z[i] ^= mask & syndromes[i - 1];
 
 		for (j = 1; j < i; ++j) {
@@ -1523,8 +1539,8 @@ static void compute_error_values(uint16_t *error_values, const uint16_t *z,
 		found = 0;
 		mask1 = (uint16_t)(-((int32_t)error[i]) >> 31); // error[i] != 0
 		for (j = 0; j < LC_HQC_PARAM_DELTA; j++) {
-			mask2 = ~((uint16_t)(-((int32_t)j ^ delta_counter) >>
-					     31)); // j == delta_counter
+			mask2 = (uint16_t)(~((uint16_t)(-((int32_t)j ^ delta_counter) >>
+					     31))); // j == delta_counter
 			beta_j[j] += mask1 & mask2 & gf_exp[i];
 			found += mask1 & mask2 & 1;
 		}
@@ -1562,8 +1578,8 @@ static void compute_error_values(uint16_t *error_values, const uint16_t *z,
 		found = 0;
 		mask1 = (uint16_t)(-((int32_t)error[i]) >> 31); // error[i] != 0
 		for (j = 0; j < LC_HQC_PARAM_DELTA; j++) {
-			mask2 = ~((uint16_t)(-((int32_t)j ^ delta_counter) >>
-					     31)); // j == delta_counter
+			mask2 = (uint16_t)(~((uint16_t)(-((int32_t)j ^ delta_counter) >>
+					     31))); // j == delta_counter
 			error_values[i] += mask1 & mask2 & e_j[j];
 			found += mask1 & mask2 & 1;
 		}
