@@ -26,6 +26,13 @@
 extern "C" {
 #endif
 
+/* First prio is to ensure the algorithm selectors are run */
+#define LC_INIT_PRIO_ALGO (101)
+/* Second prio is to make the library available */
+#define LC_INIT_PRIO_LIBRARY (103)
+/* Last prio is to run the FIPS integrity test */
+#define LC_INIT_PRIO_FIPS (102)
+
 void ascon_fastest_impl(void);
 void sha256_fastest_impl(void);
 void sha512_fastest_impl(void);
@@ -34,6 +41,7 @@ void aes_fastest_impl(void);
 void kyber_riscv_rvv_selector(void);
 void secure_execution_linux(void);
 void chacha20_fastest_impl(void);
+void lc_activate_library(void);
 
 #ifdef __cplusplus
 }
