@@ -20,7 +20,7 @@
 #ifndef LEFT_ENCODE_H
 #define LEFT_ENCODE_H
 
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
 
 #ifdef __cplusplus
 extern "C" {
