@@ -20,13 +20,14 @@
 #ifndef NULL_BUFFER_H
 #define NULL_BUFFER_H
 
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#define LC_NULL_BUFFER_SIZE 64
+/* Size of 168 is LC_SHAKE_128_SIZE_BLOCK as needed by cSHAKE/KMAC */
+#define LC_NULL_BUFFER_SIZE 168
 extern const uint8_t null_buffer[];
 
 extern volatile int16_t optimization_blocker_int16;
