@@ -17,22 +17,14 @@
  * DAMAGE.
  */
 
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
 #include "lc_status.h"
+#include "test_helper_common.h"
 #include "visibility.h"
 
 static int status_tester(void)
 {
-	char status[900];
-	int ret = 0;
-
-	memset(status, 0, sizeof(status));
-	lc_status(status, sizeof(status));
-	if (strlen(status) == 0)
-		ret = 1;
-	printf("Status information from leancrypto:\n%s", status);
-
-	return ret;
+	return test_print_status();
 }
 
 LC_TEST_FUNC(int, main, int argc, char *argv[])
