@@ -19,6 +19,8 @@
 
 #include "compare.h"
 #include "lc_kmac.h"
+#include "ret_checkers.h"
+#include "test_helper_common.h"
 #include "visibility.h"
 
 #include "sha3_c.h"
@@ -769,7 +771,7 @@ static int _kmac_128_tester(const struct lc_hash *cshake_128, const char *name)
 	       cshake_128 == lc_cshake128_c ? "C" : "accelerated",
 	       LC_KMAC_CTX_SIZE(cshake_128));
 
-	lc_kmac_init(ctx_re, key1, sizeof(key1), cust1, sizeof(cust1));
+	CKINT(lc_kmac_init(ctx_re, key1, sizeof(key1), cust1, sizeof(cust1)));
 	lc_kmac_update(ctx_re, msg1, sizeof(msg1));
 	lc_kmac_final(ctx_re, act1, sizeof(act1));
 	ret = lc_compare(act1, exp1, sizeof(act1), "KMAC128 1");
@@ -788,13 +790,14 @@ static int _kmac_128_tester(const struct lc_hash *cshake_128, const char *name)
 	if (ret)
 		return ret;
 
-	lc_kmac_init(ctx, key2, sizeof(key2), cust2, sizeof(cust2));
+	CKINT(lc_kmac_init(ctx, key2, sizeof(key2), cust2, sizeof(cust2)));
 	lc_kmac_update(ctx, msg2, sizeof(msg2));
 	lc_kmac_final(ctx, act2, sizeof(act2));
 	ret = lc_compare(act2, exp2, sizeof(act2), "KMAC128 2");
 	lc_kmac_zero(ctx);
 
-	return ret;
+out:
+	return !!ret;
 }
 
 static int kmac_128_tester(void)
@@ -814,7 +817,15 @@ static int kmac_128_tester(void)
 
 LC_TEST_FUNC(int, main, int argc, char *argv[])
 {
+	int ret;
+
 	(void)argc;
 	(void)argv;
-	return kmac_128_tester();
+
+	ret = kmac_128_tester();
+
+	ret = test_validate_status(ret, LC_ALG_STATUS_KMAC, 1);
+	ret += test_print_status();
+
+	return ret;
 }
