@@ -21,6 +21,7 @@
 #include "lc_kmac.h"
 #include "lc_rng.h"
 #include "math_helper.h"
+#include "ret_checkers.h"
 #include "visibility.h"
 
 #include "sha3_c.h"
@@ -678,7 +679,7 @@ static int _kmac_256_xof_tester(const struct lc_hash *cshake_256,
 	if (lc_kmac_alloc(cshake_256, &ctx, LC_KMAC_FLAGS_SUPPORT_REINIT))
 		return 1;
 
-	lc_kmac_init(ctx, key1, sizeof(key1), cust1, sizeof(cust1));
+	CKINT(lc_kmac_init(ctx, key1, sizeof(key1), cust1, sizeof(cust1)));
 	lc_kmac_update(ctx, msg1, sizeof(msg1));
 	lc_kmac_final_xof(ctx, act1, sizeof(act1));
 	ret = lc_compare(act1, exp1, sizeof(act1), "KMAC256 XOF 1");
@@ -697,8 +698,8 @@ static int _kmac_256_xof_tester(const struct lc_hash *cshake_256,
 	if (ret)
 		goto out;
 
-	lc_kmac_xof(cshake_256, key2, sizeof(key2), NULL, 0, msg2, sizeof(msg2),
-		    act2, sizeof(act2));
+	CKINT(lc_kmac_xof(cshake_256, key2, sizeof(key2), NULL, 0, msg2,
+			  sizeof(msg2), act2, sizeof(act2)));
 	ret = lc_compare(act2, exp2, sizeof(act2), "KMAC256 XOF 2");
 	lc_kmac_zero(ctx);
 
