@@ -20,7 +20,7 @@
 #include <asm/simd.h>
 
 #include "cpufeatures.h"
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
 #include "visibility.h"
 
 static enum lc_cpu_features features = LC_CPU_FEATURE_UNSET;
