@@ -17,13 +17,14 @@
  * DAMAGE.
  */
 
-#include "ext_headers.h"
+#include "ext_headers_internal.h"
 
 #include "compare.h"
 #include "kyber_x25519_internal.h"
 #include "ret_checkers.h"
 #include "small_stack_support.h"
 #include "selftest_rng.h"
+#include "test_helper_common.h"
 #include "visibility.h"
 
 #if LC_KYBER_K == 2
@@ -1180,7 +1181,7 @@ static int kyber_kem_double_tester(int check)
 	LC_DECLARE_MEM(ws, struct workspace, sizeof(uint64_t));
 	LC_SELFTEST_DRNG_CTX_ON_STACK(selftest_rng);
 
-	CKINT(lc_kyber_x25519_keypair(&ws->pk, &ws->sk, selftest_rng));
+	ret = lc_kyber_x25519_keypair(&ws->pk, &ws->sk, selftest_rng);
 
 	if (check) {
 		rc += lc_compare(ws->pk.pk.pk, kyber_pk_exp.pk.pk,
@@ -1195,11 +1196,14 @@ static int kyber_kem_double_tester(int check)
 				 "X25519 sk keygen\n");
 	}
 
-	CKINT(lc_kyber_x25519_enc_kdf_internal(
-		&ws->ct, ws->ss1, sizeof(ws->ss1), &ws->pk, selftest_rng));
+	ret |= lc_kyber_x25519_enc_kdf_internal(
+		&ws->ct, ws->ss1, sizeof(ws->ss1), &ws->pk, selftest_rng);
 
-	CKINT(lc_kyber_x25519_dec_kdf(ws->ss2, sizeof(ws->ss2), &ws->ct,
-				      &ws->sk));
+	ret += lc_kyber_x25519_dec_kdf(ws->ss2, sizeof(ws->ss2), &ws->ct,
+				       &ws->sk);
+
+	if (ret)
+		goto out;
 
 	if (check) {
 		rc += lc_compare(ws->ss1, ss_exp, sizeof(ss_exp),
@@ -1210,7 +1214,7 @@ static int kyber_kem_double_tester(int check)
 
 out:
 	LC_RELEASE_MEM(ws);
-	return ret ? ret : rc;
+	return ret ? !!ret : rc;
 }
 
 static int kyber_kem_double_enc_tester(void)
@@ -1277,25 +1281,50 @@ out:
 }
 LC_TEST_FUNC(int, main, int argc, char *argv[])
 {
-	size_t count;
 	int ret = 0;
 
 	(void)argv;
 
+#ifdef LC_FIPS140_DEBUG
+	/*
+	 * Both algos are used for the random number generation as part of
+	 * the key generation. Thus we need to enable them for executing the
+	 * test.
+	 */
+	alg_status_set_result(lc_alg_status_result_passed, LC_ALG_STATUS_SHAKE);
+	alg_status_set_result(lc_alg_status_result_passed, LC_ALG_STATUS_SHA3);
+#endif
+
 	if (argc != 2)
-		return kyber_kem_double_tester(1);
-
+		ret = kyber_kem_double_tester(1);
 	else if (argv[1][0] == 'e')
-		return kyber_kem_double_enc_tester();
-
+		ret = kyber_kem_double_enc_tester();
 	else if (argv[1][0] == 'd')
-		return kyber_kem_double_dec_tester();
-
+		ret = kyber_kem_double_dec_tester();
 	else if (argv[1][0] == 'k')
-		return kyber_kem_double_keygen_tester();
+		ret = kyber_kem_double_keygen_tester();
+	else {
+		unsigned int count;
 
-	for (count = 0; count < 50000; count++)
-		ret += kyber_kem_double_tester(0);
+		for (count = 0; count < 50000; count++)
+			ret += kyber_kem_double_tester(0);
+	}
+
+	/*
+	 * Only verify kyber_kem_tester_common because the other tests
+	 * disable the self tests.
+	 */
+	if (argc != 2) {
+		ret = test_validate_status(ret, LC_ALG_STATUS_MLKEM_KEYGEN, 1);
+		ret = test_validate_status(ret, LC_ALG_STATUS_MLKEM_ENC, 1);
+		ret = test_validate_status(ret, LC_ALG_STATUS_MLKEM_DEC, 1);
+#ifndef LC_FIPS140_DEBUG
+		ret = test_validate_status(ret, LC_ALG_STATUS_X25519_KEYGEN, 0);
+		ret = test_validate_status(ret, LC_ALG_STATUS_X25519_SS, 0);
+		ret = test_validate_status(ret, LC_ALG_STATUS_SHAKE, 1);
+#endif
+		ret += test_print_status();
+	}
 
 	return ret;
 }
