@@ -28,6 +28,10 @@ extern const struct lc_sym *lc_aes_cbc_c;
 extern const struct lc_sym *lc_aes_ctr_c;
 extern const struct lc_sym *lc_aes_kw_c;
 extern const struct lc_sym *lc_aes_c;
+extern const struct lc_sym *lc_aes_xts_c;
+
+/* Maximum size of the AES context */
+#define LC_AES_C_MAX_BLOCK_SIZE (244)
 
 #ifdef __cplusplus
 }
