@@ -29,6 +29,10 @@ extern const struct lc_sym *lc_aes_ctr_riscv64;
 extern const struct lc_sym *lc_aes_kw_riscv64;
 extern const struct lc_sym *lc_aes_riscv64;
 extern const struct lc_sym *lc_aes_riscv64_enc_only;
+extern const struct lc_sym *lc_aes_xts_riscv64;
+
+/* Maximum size of the AES context */
+#define LC_AES_RISCV64_MAX_BLOCK_SIZE (244 * 2)
 
 #ifdef __cplusplus
 }
