@@ -28,6 +28,7 @@
 #include "compare.h"
 #include "lc_chacha20.h"
 #include "math_helper.h"
+#include "test_helper_common.h"
 #include "ret_checkers.h"
 #include "visibility.h"
 
@@ -317,7 +318,7 @@ static int chacha20_enc_large(const struct lc_sym *chacha20_sym,
 	LC_SYM_CTX_ON_STACK(chacha20, chacha20_sym);
 
 	/* Encrypt */
-	lc_sym_init(chacha20);
+	CKINT(lc_sym_init(chacha20));
 	CKINT(lc_sym_setkey(chacha20, (uint8_t *)key, sizeof(key)));
 	CKINT(lc_sym_setiv(chacha20, (uint8_t *)iv, sizeof(iv)));
 	lc_sym_encrypt(chacha20, (uint8_t *)string, res, sizeof(string));
@@ -330,7 +331,7 @@ static int chacha20_enc_large(const struct lc_sym *chacha20_sym,
 	lc_sym_zero(chacha20);
 
 	/* Decrypt */
-	lc_sym_init(chacha20);
+	CKINT(lc_sym_init(chacha20));
 	CKINT(lc_sym_setkey(chacha20, (uint8_t *)key, sizeof(key)));
 	CKINT(lc_sym_setiv(chacha20, (uint8_t *)iv, sizeof(iv)));
 	lc_sym_decrypt(chacha20, res, res, sizeof(res));
@@ -343,7 +344,7 @@ static int chacha20_enc_large(const struct lc_sym *chacha20_sym,
 
 out:
 	lc_sym_zero(chacha20);
-	return ret;
+	return !!ret;
 }
 
 /* Test vector according to RFC 7539 section 2.4.2 */
@@ -381,7 +382,7 @@ static int chacha20_enc_selftest(const struct lc_sym *chacha20_sym,
 	       (unsigned int)LC_SYM_CTX_SIZE(chacha20_sym));
 
 	/* Encrypt */
-	lc_sym_init(chacha20);
+	CKINT(lc_sym_init(chacha20));
 	CKINT(lc_sym_setkey(chacha20, (uint8_t *)rfc_key, sizeof(rfc_key)));
 	CKINT(lc_sym_setiv(chacha20, (uint8_t *)rfc_iv, sizeof(rfc_iv)));
 	lc_sym_encrypt(chacha20, (uint8_t *)rfc_string, res, sizeof(res));
@@ -394,7 +395,7 @@ static int chacha20_enc_selftest(const struct lc_sym *chacha20_sym,
 	lc_sym_zero(chacha20);
 
 	/* Decrypt */
-	lc_sym_init(chacha20);
+	CKINT(lc_sym_init(chacha20));
 	CKINT(lc_sym_setkey(chacha20, (uint8_t *)rfc_key, sizeof(rfc_key)));
 	CKINT(lc_sym_setiv(chacha20, (uint8_t *)rfc_iv, sizeof(rfc_iv)));
 	lc_sym_decrypt(chacha20, res, res, sizeof(res));
@@ -407,7 +408,7 @@ static int chacha20_enc_selftest(const struct lc_sym *chacha20_sym,
 
 out:
 	lc_sym_zero(chacha20);
-	return ret;
+	return !!ret;
 }
 
 static int chacha20_stream_test(const struct lc_sym *chacha20_sym,
@@ -423,7 +424,7 @@ static int chacha20_stream_test(const struct lc_sym *chacha20_sym,
 	LC_SYM_CTX_ON_STACK(chacha20, chacha20_sym);
 
 	/* Encrypt */
-	lc_sym_init(chacha20);
+	CKINT(lc_sym_init(chacha20));
 	CKINT(lc_sym_setkey(chacha20, (uint8_t *)rfc_key, sizeof(rfc_key)));
 	CKINT(lc_sym_setiv(chacha20, (uint8_t *)rfc_iv, sizeof(rfc_iv)));
 
@@ -451,7 +452,7 @@ static int chacha20_stream_test(const struct lc_sym *chacha20_sym,
 	lc_sym_zero(chacha20);
 
 	/* Decrypt */
-	lc_sym_init(chacha20);
+	CKINT(lc_sym_init(chacha20));
 	CKINT(lc_sym_setkey(chacha20, (uint8_t *)rfc_key, sizeof(rfc_key)));
 	CKINT(lc_sym_setiv(chacha20, (uint8_t *)rfc_iv, sizeof(rfc_iv)));
 
@@ -479,7 +480,7 @@ static int chacha20_stream_test(const struct lc_sym *chacha20_sym,
 
 out:
 	lc_sym_zero(chacha20);
-	return ret;
+	return !!ret;
 }
 
 LC_TEST_FUNC(int, main, int argc, char *argv[])
@@ -496,5 +497,8 @@ LC_TEST_FUNC(int, main, int argc, char *argv[])
 	LC_EXEC_ONE_TEST(lc_chacha20_avx2)
 	LC_EXEC_ONE_TEST(lc_chacha20_avx512)
 
+	ret = test_validate_status(ret, LC_ALG_STATUS_CHACHA20, 0);
+	ret += test_print_status();
+
 	return ret;
 }
